#define SCL_PIN  2                 // SCLは2番ピンとする
#define SCL_PORT PORTD 
#define SDA_PIN  3                 // SDAは3番ピンとする
#define SDA_PORT PORTD 
// #define I2C_FASTMODE     1         // I2Cの通信速度は400KHzとする
#include <SoftI2CMaster.h>         // ソフトI2Cを利用する為のファイル


#define HDC1000_ADRS    0x80       // HDC1000のI2Cアドレス(8bit)
#define HDC1000_ID      0x1000     // HDC1000のID

// デバイスのレジスタアドレス
#define DEVICE_ID_ADRS  0xFF       // HDC1000のデバイスID格納アドレス
#define CONFIG_ADRS     0x02       // Configurationレジスタのアドレス
#define TEMP_ADRS       0x00       // 温度読出しのレジスタのアドレス
#define HUMI_ADRS       0x01       // 湿度読出しのレジスタのアドレス

// Configurationレジスタの設定
#define CONFIG_DT_H     0b00000000 // 温度(14bit)･湿度(14bit)個別に読出し
#define CONFIG_DT_L     0b00000000 // 予約の為に全て０

float Humi ;                       // 湿度の値を保存する変数
float Temp ;                       // 温度の値を保存する変数
int   Sensor_adrs ;


void setup()
{
     int ans ;

     // シリアルモニターの設定
     Serial.begin(9600) ;
     delay(5000) ;                 // 5Sしたら開始
     // ソフトＩ２Ｃの初期化
     if (i2c_init()) Serial.println("I2C Initialization normal !!") ;
     else {
          // SDA/SCLピンがLOWである(プルアップされていない)
          Serial.println("I2C Initialization Error !!") ;
     }
     // HDC1000の初期化
     ans = HDC_Init(HDC1000_ID,HDC1000_ADRS) ;
     if (ans != 0) {
          Serial.print("HDC Initialization abnormal ans=") ;
          Serial.println(ans) ;
     } else Serial.println("HDC Initialization normal !!") ;
}
void loop()
{
     int ans ;

     // HDC1000から湿度と温度を読み取る
     // 読み取った値は、Humi/Temp変数に格納される
     ans = HDC_Read() ;
     if (ans == 0) {
          Serial.print("HDC1000: ") ;
          Serial.print(Humi) ;
          Serial.print("%  ") ;
          Serial.print(Temp) ;
          Serial.println("'C") ;
     } else {
          Serial.print("Failed to read from HDC ans=") ;
          Serial.println(ans) ;
     }
     delay(1000) ;                      // １秒後に繰り返す
}
/*******************************************************************************
*  ans = HDC_Init(id, address)                                                 *
*    デバイスの初期化を行う処理                                                *
*    デバイスの動作を確かめる為にデバイスIDのチェックを行います。              *
*    温度(14bit)･湿度(14bit)を個別に読出す様に設定を初期化します。             *
*    設定データを変更する場合は、HDC.hのCONFIG_DT_Hを書き換え変更して下さい。  *
*    id      : デバイスの識別IDを指定します                                    *
*    address : デバイス(スレーブ)のI2Cアドレスを指定します                     *
*    ans     : 0=正常　                                                        *
*              2=スレーブ・アドレスを送信し、NACKを受信した                    *
*              3=データ・バイトを送信し、NACKを受信した                        *
*              6=デバイスのIDチェックエラー                                    *
*******************************************************************************/
int HDC_Init(int id ,int address)
{
     union {
          uint16_t  i ;
          uint8_t   c[2] ;
     } data ;
     int  ans ;

     Sensor_adrs = address ;

     // 電源投入後15ms待ち
     delay(15) ;
     // デバイスの識別ＩＤをチェックする処理
     ans = HDC_Receive(DEVICE_ID_ADRS,data.c) ;
     if (ans == 0) {
          if (data.i == HDC1000_ID) {
               // ＩＤは一致したならデバイスを初期化する処理
               data.c[0] = CONFIG_DT_H ;
               data.c[1] = CONFIG_DT_L ;
               HDC_Send(CONFIG_ADRS,data.c) ;
          } else ans = 6 ; // ＩＤが一致しない
     }
     return ans ;
}
/*******************************************************************************
*  ans = HDC_Read()                                                            *
*  湿度･温度を読込む処理                                                       *
*  読み込まれた湿度(％)はHumi変数に、温度値(℃)はTemp変数に其々格納されます。  *
*  この処理は約15ms程時間が掛かります。                                        *
*    ans      : 0=正常                                                         *
*               2=スレーブ・アドレスを送信し、NACKを受信した                   *
*               3=データ・バイトを送信し、NACKを受信した                       *
*******************************************************************************/
int HDC_Read()
{
     union {
          uint16_t  i ;
          uint8_t   c[2] ;
     } data ;
     int ans ;

     // 湿度データを受信する
     ans = HDC_Receive(HUMI_ADRS,data.c) ;
     if (ans == 0) {
          Humi = (float)data.i / 65536.0 ;
          Humi = Humi * 100.0 ;
          // 温度データを受信する
          ans = HDC_Receive(TEMP_ADRS,data.c) ;
          if (ans == 0) {
               Temp = (float)data.i / 65536.0 ;
               Temp = (Temp * 165.0) - 40.0 ;
          }
     }
     return ans ;
}
/*******************************************************************************
*  ans = HDC_Receive(reg_adrs,*data)                                           *
*  デバイスのレジスターからデータを読込む処理                                  *
*    reg_adrs : 読出すデータのレジスターアドレスを指定する                     *
*    *data    : 読出したデータの格納先を指定する(2byte読込む)                  *
*    ans      : 0=正常                                                         *
*               2=スレーブ・アドレスを送信し、NACKを受信した                   *
*               3=データ・バイトを送信し、NACKを受信した                       *
*******************************************************************************/
int HDC_Receive(char reg_adrs,unsigned char *data)
{
     int  ans ;

     ans = 0 ;
     if (i2c_start(Sensor_adrs|I2C_WRITE)) {      // スタートコンディションを発行する
          if (i2c_write(reg_adrs)) {              // レジスタアドレスを指定
               if ( (reg_adrs == TEMP_ADRS) || (reg_adrs == HUMI_ADRS) )  {
                    // 温度・湿度の読出しなら変換が完了するまで待つ
                    delay(7) ;
               }
               if (i2c_rep_start(Sensor_adrs|I2C_READ)) { // リピート・スタートコンディションを発行する
                    data++ ;
                    *data = (char)i2c_read(false);// HIGHバイト読出し
                    data-- ;
                    *data = (char)i2c_read(true) ;// LOWバイト読出し
               } else ans = 2 ;
          } else ans = 3 ;
     } else ans = 2 ;
     i2c_stop() ;                                 // ストップコンディションを発行する

     return ans ;
}
/*******************************************************************************
*  ans = HDC_Send(reg_adrs,*data)                                              *
*  デバイスのレジスターへデータを書込む処理                                    *
*    reg_adrs : 書出すデータのレジスターアドレスを指定する                     *
*    *data    : 書出すデータの格納先を指定する(2byte書込む)                    *
*    ans      : 0=正常                                                         *
*               2=スレーブ・アドレスを送信し、NACKを受信した                   *
*               3=データ・バイトを送信し、NACKを受信した                       *
*******************************************************************************/
int HDC_Send(char reg_adrs,unsigned char *data)
{
     int  ans , i ;

     ans = 0 ;
     if (i2c_start(Sensor_adrs|I2C_WRITE)) { // スタートコンディションを発行する
          if (i2c_write(reg_adrs)) {         // レジスタアドレスを指定
               for (i=0 ; i<2 ; i++) {
                    i2c_write(*data) ;       // 2byte書き込む(送信する)
                    data++ ;
               }
          } else ans = 3 ;
     } else ans = 2 ;
     i2c_stop() ;                            // ストップコンディションを発行する
     return ans ;
}
